//------------------------------------------------------------------------------
// File       : gig_eth_pcs_pma_sgmii_v10_3_example_design.v
// Author     : Xilinx Inc.
//------------------------------------------------------------------------------
// (c) Copyright 2009 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
// 
// 
//------------------------------------------------------------------------------
// Description: This is the top level Verilog example design for the
//              Ethernet SGMII core.  The block level wrapper for the
//              core is instantiated and the RocketIO clock circuitry is
//              created.  Additionally, the I/O of the GMII-style
//              interface is provided with IOB flip-flops (infered)
//              which enables this example design to be implemented
//              using the Xilinx tools.
//
//           * Please refer to the Getting Started User Guide for
//             details of the example design file hierarchy.



`timescale 1 ps/1 ps

//------------------------------------------------------------------------------
// The module declaration for the top level of the example design.
//------------------------------------------------------------------------------

module gig_eth_pcs_pma_sgmii_v10_3_example_design
   (
      //------------------------------------------------------------------------
      // Core connected to GTP0
      //------------------------------------------------------------------------

      // GMII Interface
      //---------------
      output           sgmii_clk0,            // Clock for client MAC (125Mhz, 12.5MHz or 1.25MHz).
      input      [7:0] gmii_txd0,             // Transmit data from client MAC.
      input            gmii_tx_en0,           // Transmit control signal from client MAC.
      input            gmii_tx_er0,           // Transmit control signal from client MAC.
      output reg [7:0] gmii_rxd0,             // Received Data to client MAC.
      output reg       gmii_rx_dv0,           // Received control signal to client MAC.
      output reg       gmii_rx_er0,           // Received control signal to client MAC.

      // Management: MDIO Interface
      //---------------------------
      input            mdc0,                  // Management Data Clock
      input            mdio0_i,               // Management Data In
      output           mdio0_o,               // Management Data Out
      output           mdio0_t,               // Management Data Tristate
      input      [4:0] phyad0,                // Port address for MDIO.

      output           an_interrupt0,         // Interrupt to processor to signal that Auto-Negotiation has completed
      // Dynamic switching
      //------------------
      input      [8:0] link_timer_basex0,     // Programmable Auto-Negotiation Link Timer Control for 1000BASE-X standard
      input      [8:0] link_timer_sgmii0,     // Programmable Auto-Negotiation Link Timer Control for SGMII standard
      input            basex_or_sgmii0,       // Reset default for 1000BASE-X (0) or SGMII standard (1)

      // General IO's
      //-------------
      output     [7:0] status_vector0,        // Synchronisation status.
      input            reset0,                // Asynchronous reset for entire core.
      input            signal_detect0,        // Input from PMD to indicate presence of optical input.

      // Speed Control
      //--------------
      input            speed0_is_10_100,      // Core should operate at either 10Mbps or 100Mbps speeds
      input            speed0_is_100,         // Core should operate at 100Mbps speed


      //------------------------------------------------------------------------
      // RocketIO interfaces
      //------------------------------------------------------------------------

      input            brefclk_p,             // Differential +ve of reference clock for RocketIO: 125MHz, very high quality
      input            brefclk_n,             // Differential -ve of reference clock for RocketIO: 125MHz, very high quality

      output           txp0,                  // Differential +ve of serial transmission from PMA to PMD.
      output           txn0,                  // Differential -ve of serial transmission from PMA to PMD.
      input            rxp0,                  // Differential +ve for serial reception from PMD to PMA.
      input            rxn0,                  // Differential -ve for serial reception from PMD to PMA.

      output           txp1,                  // Differential +ve of serial transmission from PMA to PMD.
      output           txn1,                  // Differential -ve of serial transmission from PMA to PMD.
      input            rxp1,                  // Differential +ve for serial reception from PMD to PMA.
      input            rxn1                   // Differential -ve for serial reception from PMD to PMA.
   );



   //---------------------------------------------------------------------------
   // internal signals used in this top level example design.
   //---------------------------------------------------------------------------

   // clock/reset generation signals
   wire         clkin;                    // RocketIO 125MHz clock, very high quality.
   wire         userclk2;                 // 125MHz reference clock derived from RocketIO clock circuitry.
   wire         not_userclk2;             // Inverted form of userclk2.
   wire         gtpclkout;                // RocketIO output clock made available to the FPGA fabric.
   wire         gtpclkout_bufio2;         // gtpclkout is routed through a BUFIO2 clock buffer


   // Signals for the core connected to GTP0
   //----------------------------------------

   wire         gtpreset0;                // System reset for RocketIO.
   wire         sgmii_clk_r0;             // Clock to client MAC (125MHz, 12.5MHz or 1.25MHz) (to rising edge DDR).
   wire         sgmii_clk_f0;             // Clock to client MAC (125MHz, 12.5MHz or 1.25MHz) (to falling edge DDR).
   wire         gmii_isolate0;            // internal gmii_isolate signal.
   reg   [7:0]  gmii_txd_int0;            // internal gmii_txd signal.
   reg          gmii_tx_en_int0;          // internal gmii_tx_en signal.
   reg          gmii_tx_er_int0;          // internal gmii_tx_er signal.
   wire  [7:0]  gmii_rxd_int0;            // internal gmii_rxd signal.
   wire         gmii_rx_dv_int0;          // internal gmii_rx_dv signal.
   wire         gmii_rx_er_int0;          // internal gmii_rx_er signal.




   //---------------------------------------------------------------------------
   // Spartan-6 Rocket I/O Clock Management
   //---------------------------------------------------------------------------

   // NOTE: BREFCLK circuitry for the Rocket I/O requires the use of a
   // 125MHz differential input clock.  clkin is routed to the RocketIO
   // pair.

   IBUFDS clkingen (
      .I  (brefclk_p),
      .IB (brefclk_n),
      .O  (clkin)
   );


   // gtpclkout (125MHz) is made avaiable by the RocketIO to the FPGA
   // fabric. This is routed to a BUFIO2 before being placed onto global clock
   // routing where it is then used for RocketIO TXUSRCLK2/RXUSRCLK2 and used
   // to clock all Ethernet core logic.

   // Route through a BUFIO2
   BUFIO2 # (
      .DIVIDE        (1),
      .DIVIDE_BYPASS ("TRUE")
   ) bufio2_clk125m  (
      .DIVCLK        (gtpclkout_bufio2),
      .I             (gtpclkout),
      .IOCLK         (),
      .SERDESSTROBE  ()
   );


   // Route through a BUFG
   BUFG bufg_clk125m (
      .I   (gtpclkout_bufio2),
      .O   (userclk2)
   );


   // An inverted version of userclk2 is created.
   INV invert_userclk2 
   (
      .I  (userclk2),
      .O  (not_userclk2)
   );


   //---------------------------------------------------------------------------
   // Spartan-6 Rocket System Reset
   //---------------------------------------------------------------------------

   // RocketIO 0
   reset_sync gtpreset0_gen (
      .clk       (userclk2),
      .reset_in  (reset0),
      .reset_out (gtpreset0)
   );


   //---------------------------------------------------------------------------
   // Instantiate the Core Block (core wrapper).
   //---------------------------------------------------------------------------
   gig_eth_pcs_pma_sgmii_v10_3_block #
   (
      // Simulation attribute: this setting does not affect the hardware
      // It is a Smartmodel setting only.  Setting it to 1 reduces the
      // simulation time required for the GTP to intialise.
      .SIM_GTPRESET_SPEEDUP  (1)
   )
   core_wrapper
   (
      .gtpclkout             (gtpclkout),

      .gtpreset0             (gtpreset0),
      .sgmii_clk_r0          (sgmii_clk_r0),
      .sgmii_clk_f0          (sgmii_clk_f0),
      .sgmii_clk_en0         (),
      .gmii_txd0             (gmii_txd_int0),
      .gmii_tx_en0           (gmii_tx_en_int0),
      .gmii_tx_er0           (gmii_tx_er_int0),
      .gmii_rxd0             (gmii_rxd_int0),
      .gmii_rx_dv0           (gmii_rx_dv_int0),
      .gmii_rx_er0           (gmii_rx_er_int0),
      .gmii_isolate0         (gmii_isolate0),
      .mdc0                  (mdc0),
      .mdio0_i               (mdio0_i),
      .mdio0_o               (mdio0_o),
      .mdio0_t               (mdio0_t),
      .phyad0                (phyad0),
      .an_interrupt0         (an_interrupt0),
      .link_timer_basex0     (link_timer_basex0),
      .link_timer_sgmii0     (link_timer_sgmii0),
      .basex_or_sgmii0       (basex_or_sgmii0),
      .status_vector0        (status_vector0),
      .reset0                (reset0),
      .signal_detect0        (signal_detect0),
      .speed0_is_10_100      (speed0_is_10_100),
      .speed0_is_100         (speed0_is_100),

      .clkin                 (clkin),
      .userclk2              (userclk2),

      .txp0                  (txp0),
      .txn0                  (txn0),
      .rxp0                  (rxp0),
      .rxn0                  (rxn0),
      .txp1                  (txp1),
      .txn1                  (txn1),
      .rxp1                  (rxp1),
      .rxn1                  (rxn1)

   );



   //---------------------------------------------------------------------------
   // GMII logic for the core connected to GTP0
   //---------------------------------------------------------------------------


   // GMII transmitter data logic
   //----------------------------

   // Drive input GMII signals through IOB input flip-flops (inferred).
   always @ (posedge userclk2)
   begin
      gmii_txd_int0    <= gmii_txd0;
      gmii_tx_en_int0  <= gmii_tx_en0;
      gmii_tx_er_int0  <= gmii_tx_er0;
   end



   // SGMII clock logic
   //------------------

   // All GMII transmitter input signals must be synchronous to this
   // clock.

   // All GMII receiver output signals are synchrounous to this clock.

   // This instantiates a DDR output register.  This is a nice way to
   // drive the output clock since the clock-to-PAD delay will the
   // same as that of data driven from an IOB Ouput flip-flop.

   ODDR2 sgclk_ddr_iob0 (
      .Q  (sgmii_clk0),
      .C0 (userclk2),
      .C1 (not_userclk2),
      .CE (1'b1),
      .D0 (sgmii_clk_r0),
      .D1 (sgmii_clk_f0),
      .R  (1'b0),
      .S  (1'b0)
   );



   // GMII receiver data logic
   //-------------------------

   // Drive input GMII signals through IOB output flip-flops (inferred).
   always @ (posedge userclk2)
   begin
      gmii_rxd0    <= gmii_rxd_int0;
      gmii_rx_dv0  <= gmii_rx_dv_int0;
      gmii_rx_er0  <= gmii_rx_er_int0;
   end



endmodule // gig_eth_pcs_pma_sgmii_v10_3_example_design

