//------------------------------------------------------------------------------
// File       : transceiver_A.v
// Author     : Xilinx Inc.
//------------------------------------------------------------------------------
// (c) Copyright 2009 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES.
//
//

//------------------------------------------------------------------------------
// Description:  This is the VHDL instantiation of a Spartan-6 GTP
//               RocketIO tile for Gigabit Ethernet. The
//               instantiated component is generated by the GTP Wizard.
//------------------------------------------------------------------------------


`timescale 1 ps/1 ps

module transceiver_A #
(
   // Set to 1 to Speed up the GTP simulation
   parameter SIM_GTPRESET_SPEEDUP = 0
)
(
   output         gtpclkout,
   input          gtpclkin,

   // I/O for GTP0
   input          gtpreset0,
   output         plllkdet0,
   output         resetdone0,
   input          enablealign0,
   input          powerdown0,
   input          loopback0,
   output         rxchariscomma0,
   output         rxcharisk0,
   output [2:0]   rxclkcorcnt0,
   output [7:0]   rxdata0,
   output         rxdisperr0,
   output         rxnotintable0,
   output         rxrundisp0,
   output         rxbuferr0,
   input          rxusrclk0,
   input          rxusrclk20,
   input          rxreset0,
   input          txchardispmode0,
   input          txchardispval0,
   input          txcharisk0,
   input [7:0]    txdata0,
   output         txbuferr0,
   input          txusrclk0,
   input          txusrclk20,
   input          txreset0,

   // Transceiver serial I/O
   output         txn0,
   output         txp0,
   input          rxn0,
   input          rxp0,

   output         txn1,
   output         txp1,
   input          rxn1,
   input          rxp1,

   // RocketIO pair shared clock signals
   input          clkin
   );



   //---------------------------------------------------------------------------
   // Signal declarations for GTP
   //---------------------------------------------------------------------------

   // (* ASYNC_REG = "TRUE" *) attributes will stop timing errors being reported
   // in back annotated SDF simulation for signals crossing clock boundaries.

   wire [1:0]     gtpclkout_i;

   (* ASYNC_REG = "TRUE" *)
   reg  [3:0]     reset0_r;

   wire           gtpreset0_i;

   wire [1:0]     rxbufstatus_float0;
   wire           txbufstatus_float0;

   // The following GTP0 inputs will be registered to ease Spartan-6 placement
   reg            txchardispmode_reg0;
   reg            txchardispval_reg0;
   reg            txcharisk_reg0;
   reg  [7:0]     txdata_reg0;



   //---------------------------------------------------------------------------
   // Signal Declarations for GTP => FPGA fabric Rx Elastic Buffer
   //---------------------------------------------------------------------------

   // GTP0
   wire           rxrecclk0;
   wire           rxrecclk_bufg0;
   wire           rxchariscomma_rec0;
   wire           rxcharisk_rec0;
   wire           rxdisperr_rec0;
   wire           rxnotintable_rec0;
   wire           rxrundisp_rec0;
   wire [7:0]     rxdata_rec0;
   wire           rxbuferr_gtp0;
   wire           rxbuferr_gtp0_reg;
   wire           rxbuferr_fabric0;

   reg            rxchariscomma_rec_reg0;
   reg            rxcharisk_rec_reg0;
   reg            rxdisperr_rec_reg0;
   reg            rxnotintable_rec_reg0;
   reg            rxrundisp_rec_reg0;
   reg  [7:0]     rxdata_rec_reg0;


   //---------------------------------------------------------------------------
   // Reclocking Logic
   //---------------------------------------------------------------------------

   // GTP0
   wire           enablealign_rec0;
   wire           rxreset_rec0;




   // Assign internal signal to output
   assign gtpclkout = gtpclkout_i[0];


   //---------------------------------------------------------------------------
   // Full System GTP reset circuitry for GTP0
   //---------------------------------------------------------------------------

   // Create a reset pulse of a decent length
   always@(posedge gtpreset0, posedge gtpclkin)
   begin
     if (gtpreset0 == 1'b1)
     begin
       reset0_r <= 4'b1111;
     end
     else
     begin
       reset0_r <= {reset0_r[2:0], gtpreset0};
     end
   end

   assign gtpreset0_i = reset0_r[3];


   //---------------------------------------------------------------------------
   // Regsiter GTP0 signals to ease Spartan-6 placement
   //---------------------------------------------------------------------------

   always @(posedge txusrclk20)
   begin
     if (txreset0) begin
       txchardispmode_reg0  <= 1'b0;
       txchardispval_reg0   <= 1'b0;
       txcharisk_reg0       <= 1'b0;
       txdata_reg0          <= 8'b0;
     end

     else begin
       txchardispmode_reg0  <= txchardispmode0;
       txchardispval_reg0   <= txchardispval0;
       txcharisk_reg0       <= txcharisk0;
       txdata_reg0          <= txdata0;
     end
   end



   //---------------------------------------------------------------------------
   // Component instantiation for the RocketIO GTP file
   // (generated by the GTP Wizard)
   //---------------------------------------------------------------------------

    GTP_WRAPPER #
     (
        // Simulation attributes
        .WRAPPER_SIM_GTPRESET_SPEEDUP      (SIM_GTPRESET_SPEEDUP),
        .WRAPPER_SIMULATION                (0)
     )
     GTP_1000X
     (
        // GTP0 is used: it is connected to an instance of the core

        .TILE0_REFCLKOUT0_OUT              (),
        .TILE0_GTPCLKOUT0_OUT              (gtpclkout_i),
        .TILE0_PLLLKDET0_OUT               (plllkdet0),
        .TILE0_CLK00_IN                    (clkin),
        .TILE0_GTPRESET0_IN                (gtpreset0_i),
        .TILE0_RESETDONE0_OUT              (),
        .TILE0_RXBUFSTATUS0_OUT            ({rxbuferr_gtp0, rxbufstatus_float0}),
        .TILE0_RXCHARISCOMMA0_OUT          (rxchariscomma_rec0),
        .TILE0_RXCHARISK0_OUT              (rxcharisk_rec0),
        .TILE0_RXDATA0_OUT                 (rxdata_rec0),
        .TILE0_RXDISPERR0_OUT              (rxdisperr_rec0),
        .TILE0_RXNOTINTABLE0_OUT           (rxnotintable_rec0),
        .TILE0_RXRECCLK0_OUT               (rxrecclk0),
        .TILE0_RXRUNDISP0_OUT              (rxrundisp_rec0),
        .TILE0_TXBUFSTATUS0_OUT            ({txbuferr0, txbufstatus_float0}),
        .TILE0_TXN0_OUT                    (txn0),
        .TILE0_TXP0_OUT                    (txp0),
        .TILE0_TXOUTCLK0_OUT               (),
        .TILE0_LOOPBACK0_IN                ({2'b0, loopback0}),
        .TILE0_RXPOWERDOWN0_IN             ({ 2{powerdown0} }),
        .TILE0_TXPOWERDOWN0_IN             ({ 2{powerdown0} }),
        .TILE0_RXBUFRESET0_IN              (rxreset_rec0),
        .TILE0_RXENMCOMMAALIGN0_IN         (enablealign_rec0),
        .TILE0_RXENPCOMMAALIGN0_IN         (enablealign_rec0),
        .TILE0_RXN0_IN                     (rxn0),
        .TILE0_RXP0_IN                     (rxp0),
        .TILE0_RXRESET0_IN                 (rxreset_rec0),
        .TILE0_RXUSRCLK0_IN                (rxrecclk_bufg0),
        .TILE0_RXUSRCLK20_IN               (rxrecclk_bufg0),
        .TILE0_TXCHARDISPMODE0_IN          (txchardispmode_reg0),
        .TILE0_TXCHARDISPVAL0_IN           (txchardispval_reg0),
        .TILE0_TXCHARISK0_IN               (txcharisk_reg0),
        .TILE0_TXDATA0_IN                  (txdata_reg0),
        .TILE0_TXRESET0_IN                 (txreset0),
        .TILE0_TXUSRCLK0_IN                (txusrclk0),
        .TILE0_TXUSRCLK20_IN               (txusrclk20),

        // GTP1 is currently unused

        .TILE0_REFCLKOUT1_OUT              (),
        .TILE0_GTPCLKOUT1_OUT              (),
        .TILE0_PLLLKDET1_OUT               (),
        .TILE0_CLK01_IN                    (1'b0),
        .TILE0_GTPRESET1_IN                (1'b0),
        .TILE0_RESETDONE1_OUT              (),
        .TILE0_RXBUFSTATUS1_OUT            (),
        .TILE0_RXCHARISCOMMA1_OUT          (),
        .TILE0_RXCHARISK1_OUT              (),
        .TILE0_RXDATA1_OUT                 (),
        .TILE0_RXDISPERR1_OUT              (),
        .TILE0_RXNOTINTABLE1_OUT           (),
        .TILE0_RXRECCLK1_OUT               (),
        .TILE0_RXRUNDISP1_OUT              (),
        .TILE0_TXBUFSTATUS1_OUT            (),
        .TILE0_TXN1_OUT                    (txn1),
        .TILE0_TXP1_OUT                    (txp1),
        .TILE0_TXOUTCLK1_OUT               (),
        .TILE0_LOOPBACK1_IN                (3'b0),
        .TILE0_RXPOWERDOWN1_IN             (2'b11),
        .TILE0_TXPOWERDOWN1_IN             (2'b11),
        .TILE0_RXBUFRESET1_IN              (1'b0),
        .TILE0_RXENMCOMMAALIGN1_IN         (1'b0),
        .TILE0_RXENPCOMMAALIGN1_IN         (1'b0),
        .TILE0_RXN1_IN                     (rxn1),
        .TILE0_RXP1_IN                     (rxp1),
        .TILE0_RXRESET1_IN                 (1'b0),
        .TILE0_RXUSRCLK1_IN                (1'b0),
        .TILE0_RXUSRCLK21_IN               (1'b0),
        .TILE0_TXCHARDISPMODE1_IN          (1'b0),
        .TILE0_TXCHARDISPVAL1_IN           (1'b0),
        .TILE0_TXCHARISK1_IN               (1'b0),
        .TILE0_TXDATA1_IN                  (8'b0),
        .TILE0_TXRESET1_IN                 (1'b0),
        .TILE0_TXUSRCLK1_IN                (1'b0),
        .TILE0_TXUSRCLK21_IN               (1'b0)
     );



   //---------------------------------------------------------------------------
   // Instantiation of the FPGA Fabric Receiver Elastic Buffer
   // connected to RocketIO GTP0 (in use)
   //---------------------------------------------------------------------------


   // Place the Rx recovered clock on a Global Clock Buffer
   BUFG bufg_rxrecclk0 (
      .I           (rxrecclk0),
      .O           (rxrecclk_bufg0)
   );



   // Register GTP0 Rx output signals to ease Spartan-6 timing
   always @(posedge rxrecclk_bufg0)
   begin
     if (rxreset_rec0) begin
       rxchariscomma_rec_reg0   <= 1'b0;
       rxcharisk_rec_reg0       <= 1'b0;
       rxdata_rec_reg0          <= 8'b0;
       rxdisperr_rec_reg0       <= 1'b0;
       rxnotintable_rec_reg0    <= 1'b0;
       rxrundisp_rec_reg0       <= 1'b0;
     end

     else begin
       rxchariscomma_rec_reg0   <= rxchariscomma_rec0;
       rxcharisk_rec_reg0       <= rxcharisk_rec0;
       rxdata_rec_reg0          <= rxdata_rec0;
       rxdisperr_rec_reg0       <= rxdisperr_rec0;
       rxnotintable_rec_reg0    <= rxnotintable_rec0;
       rxrundisp_rec_reg0       <= rxrundisp_rec0;
     end
   end



   // Now instantiate the elastic buffer to cross clock domains onto RXUSRCLK2
   rx_elastic_buffer clock_correction_A (

      // Signals received from the RocketIO on RXRECCLK.

      .rxrecclk                     (rxrecclk_bufg0),
      .rxrecreset                   (rxreset_rec0),
      .rxchariscomma_rec            (rxchariscomma_rec_reg0),
      .rxcharisk_rec                (rxcharisk_rec_reg0),
      .rxdisperr_rec                (rxdisperr_rec_reg0),
      .rxnotintable_rec             (rxnotintable_rec_reg0),
      .rxrundisp_rec                (rxrundisp_rec_reg0),
      .rxdata_rec                   (rxdata_rec_reg0),

      // Signals reclocked onto RXUSRCLK2.

      .rxusrclk2                    (rxusrclk20),
      .rxreset                      (rxreset0),
      .rxchariscomma_usr            (rxchariscomma0),
      .rxcharisk_usr                (rxcharisk0),
      .rxdisperr_usr                (rxdisperr0),
      .rxnotintable_usr             (rxnotintable0),
      .rxrundisp_usr                (rxrundisp0),
      .rxclkcorcnt_usr              (rxclkcorcnt0),
      .rxbuferr                     (rxbuferr_fabric0),
      .rxdata_usr                   (rxdata0)
   );



   // Reclock the GTP buffer error signal from RXRECCLK onto RXUSRCLK2
   reset_sync gen_rxbuferr_gtp0 (
      .clk       (rxusrclk20),
      .reset_in  (rxbuferr_gtp0),
      .reset_out (rxbuferr_gtp0_reg)
   );


   // A buffer error can arrive from two sources!
   assign rxbuferr0 = rxbuferr_gtp0_reg || rxbuferr_fabric0;



   //---------------------------------------------------------------------------
   // Rexclocking Logic used for GTP0
   //---------------------------------------------------------------------------


   // Resynchronise the rxreset0 onto the RXRECCLK1 domain
   reset_sync rxrstreclock0 (
      .clk       (rxrecclk_bufg0),
      .reset_in  (rxreset0),
      .reset_out (rxreset_rec0)
   );



   // Re-align signals from the RXUSRCLK2 domain into the RXRECCLK1 domain
   sync_block rxrecclkreclock0 (
     .clk       (rxrecclk_bufg0),
     .data_in   (enablealign0),
     .data_out  (enablealign_rec0)
   );



endmodule

