//------------------------------------------------------------------------------
// File       : gig_eth_pcs_pma_sgmii_v11_5_mod.v
// Author     : Xilinx Inc.
//------------------------------------------------------------------------------
// (c) Copyright 2002-2008 Xilinx, Inc. All rights reserved.
//
// This file contains confidential and proprietary information
// of Xilinx, Inc. and is protected under U.S. and
// international copyright and other intellectual property
// laws.
//
// DISCLAIMER
// This disclaimer is not a license and does not grant any
// rights to the materials distributed herewith. Except as
// otherwise provided in a valid license issued to you by
// Xilinx, and to the maximum extent permitted by applicable
// law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
// WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
// AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
// BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
// INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
// (2) Xilinx shall not be liable (whether in contract or tort,
// including negligence, or under any other theory of
// liability) for any loss or damage of any kind or nature
// related to, arising under or in connection with these
// materials, including for any direct, or any indirect,
// special, incidental, or consequential loss or damage
// (including loss of data, profits, goodwill, or any type of
// loss or damage suffered as a result of any action brought
// by a third party) even if such damage or loss was
// reasonably foreseeable or Xilinx had been advised of the
// possibility of the same.
//
// CRITICAL APPLICATIONS
// Xilinx products are not designed or intended to be fail-
// safe, or for use in any application requiring fail-safe
// performance, such as life-support or safety devices or
// systems, Class III medical devices, nuclear facilities,
// applications related to the deployment of airbags, or any
// other applications that could lead to death, personal
// injury, or severe property or environmental damage
// (individually and collectively, "Critical
// Applications"). Customer assumes the sole risk and
// liability of any use of Xilinx products in Critical
// Applications, subject only to applicable laws and
// regulations governing limitations on product liability.
//
// THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
// PART OF THIS FILE AT ALL TIMES. 
//
//
//------------------------------------------------------------------------------
// Description: This package holds the top level component declaration
// for the Ethernet 1000BASE-X PCS/PMA core.



module gig_eth_pcs_pma_sgmii_v11_5
   (
      // TBI Interface
      //--------------

      input        gtx_clk,               // Reference clock for MGT: 62.5MHz, very high quality.
      output [9:0] tx_code_group,         // The ten bit parallel transmit data presented to the PMA.
      output       loc_ref,               // Causes the TBI clock recovery to lock to pma_tx_clk.
      output       ewrap,                 // This instructs the PMA to loopback the data.
      input [9:0]  rx_code_group0,        // The ten bit parallel received on pma_rx_clk0.
      input [9:0]  rx_code_group1,        // The ten bit parallel received on pma_rx_clk1.
      input        pma_rx_clk0,           // 62.5MHz rx clock that the protocol to latch odd code-groups.
      input        pma_rx_clk1,           // 62.5MHz rx clock that the protocol to even odd code-groups.
      output       en_cdet,               // Enables the PMA sublayer to perform the code-group alignment function on a comma.

      // GMII Interface (MAC <=> PCS)
      //-----------------------------

      input [7:0]  gmii_txd,              // Transmit data from client MAC.
      input        gmii_tx_en,            // Transmit control signal from client MAC.
      input        gmii_tx_er,            // Transmit control signal from client MAC.
      output [7:0] gmii_rxd,              // Received Data to client MAC.
      output       gmii_rx_dv,            // Received control signal to client MAC.
      output       gmii_rx_er,            // Received control signal to client MAC.
      output       gmii_isolate,          // Tristate control to electrically isolate GMII.

      // MDIO Interface
      //---------------

      input        mdc,                   // Management Data Clock
      input        mdio_in,               // Management Data In
      output       mdio_out,              // Management Data Out
      output       mdio_tri,              // Management Data Tristate
      input [4:0]  phyad,                 // Port address to for MDIO to recognise.
      input [4:0]  configuration_vector,  // Alternative to MDIO interface.
      input        configuration_valid,   // Validation signal for Config vector

      // General IO's
      //-------------
      input [15:0] an_adv_config_vector, // Alternate interface to program REG4 (AN ADV)
      input        an_adv_config_val,    // Validation signal for AN ADV
      input        an_restart_config,    // Alternate signal to modify AN restart bit in REG0
      input        basex_or_sgmii,        // Reset default for 1000BASE-X (0) or SGMII standard (1)
      input [8:0]  link_timer_basex,      // Programmable Auto-Negotiation Link Timer Control for 1000BASE-X standard
      input [8:0]  link_timer_sgmii,      // Programmable Auto-Negotiation Link Timer Control for SGMII standard
      output       an_interrupt,          // Interrupt to processor to signal that Auto-Negotiation has completed
      output [15:0] status_vector,         // Core status.
      input        reset,                 // Asynchronous reset for entire core.
      input        signal_detect          // Input from PMD to indicate presence of optical input.
   );

endmodule // gig_eth_pcs_pma_sgmii_v11_5

