/*******************************************************************************
*                                                                              *
* System      : BELLE-II ARICH merger module                                   *
* Module      : JTAG_IF                                                        *
* Version     : v 0.1.0 2013/12/02                                             *
*                                                                              *
* Description : JTAG control                                                   *
*                                                                              *
* Designer    : Tomohisa Uchida                                                *
*                                                                              *
*                Copyright (c) 2013 Tomohisa Uchida                            *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/

`include "../CoreGen/jtag_rx_fifo_2048.v"

module JTAG_IF(
    CLK                 ,   // in   : Clock
    RST                 ,   // in   : System reset
    // JTAG
    POUT                    ,
	 RD_ENB					,
	 JTAG_FC					,
	 JTAGENB_OUT			,
	 RD_ENB_OFF				,
    // Slow control I/F
    LOC_ENB             ,   // in   : JTAG I/F enable for each FEEs
    LOC_FULL            ,   // out  : 
	 LOC_PFULL            ,   // out  : 
    LOC_WE              ,   // in   : Write enable
    LOC_WD                  // in   : Write data[7:0]
);
    
    parameter [7:0] defFREQ = 8'd19;   // (Sys. clk. freq. - 2)/2
    
//-------- Input/Output -------------
    input           CLK             ;
    input           RST             ;
    
    output [31:0]       POUT    ;
	 output				JTAGENB_OUT;
	 output				RD_ENB_OFF;
	 
	 input				RD_ENB		;
	 
	 input	[7:0]	  JTAG_FC			;
    
    input   [5:0]   LOC_ENB         ;
    output          LOC_FULL        ;
	 output          LOC_PFULL        ;
    input   		  LOC_WE          ;
    input   [7:0]   LOC_WD          ;
    
//------------------------------------------------------------------------------
//  JTAG
//------------------------------------------------------------------------------
	
    reg             jtagEnb             ;
    reg     [7:0]   usTim               ;
	 
	 wire		[7:0]	  exLOC_WD				;
	 
	 assign exLOC_WD[0] = LOC_WD[7];
	 assign exLOC_WD[1] = LOC_WD[6];
	 assign exLOC_WD[2] = LOC_WD[5];
	 assign exLOC_WD[3] = LOC_WD[4];
	 assign exLOC_WD[4] = LOC_WD[3];
	 assign exLOC_WD[5] = LOC_WD[2];
	 assign exLOC_WD[6] = LOC_WD[1];
	 assign exLOC_WD[7] = LOC_WD[0];
	 
	 wire		[7:0]	  selJTAG_FC;
	 
	 assign selJTAG_FC[7:0] = (JTAG_FC[7:0] == 8'h00) ? defFREQ[7:0] : JTAG_FC[7:0] - 8'd1;
	 
    always @(posedge CLK or posedge RST) begin
        if(RST)begin
            {jtagEnb,usTim[7:0]} <= 9'd0;
        end else begin
            {jtagEnb,usTim[7:0]} <= (jtagEnb ? {1'b0,selJTAG_FC[7:0]} : {jtagEnb,usTim[7:0]} - 9'd1);
        end
    end
    
	 wire		[3:0]   FIFO_WE			;
	 
	 assign FIFO_WE[0] = LOC_WE & LOC_ENB[0];
	 assign FIFO_WE[1] = LOC_WE & LOC_ENB[1];
	 assign FIFO_WE[2] = LOC_WE & LOC_ENB[2];
	 assign FIFO_WE[3] = LOC_WE & LOC_ENB[3];
	 
	 assign JTAGENB_OUT = jtagEnb;
	 
	 
    wire            RX_FIFO_RE      ;
	 wire       	  RX_FIFO_VAL     ;
	 wire				  RX_FIFO_EMPTY	;
	 
    wire    [7:0]   RX_FIFO_RD0      ;
	 wire    [7:0]   RX_FIFO_RD1      ;
	 wire    [7:0]   RX_FIFO_RD2      ;
	 wire    [7:0]   RX_FIFO_RD3      ;
	 
    wire    [3:0]   andRX_FIFO_VAL     ;
    wire    [3:0]   orRX_FIFO_EMPTY   ;
    
	 wire    [3:0]   orLOC_FULL				;
	 wire    [3:0]   orLOC_PFULL				;
/*
    jtag_rx_fifo    JTAG_FIFO0 (
        .clk        (CLK            ), // input clk
        .rst        (RST            ), // input rst
        .full       (), // output full
		  .almost_full(orLOC_FULL[0]),
        .prog_full  (orLOC_PFULL[0]       ), // output prog_full
        .wr_en      (FIFO_WE[0]         ), // input wr_en
        .din        (exLOC_WD[7:0]    ), // input [7 : 0] din
        .empty      (orRX_FIFO_EMPTY[0]  ), // output empty
        .rd_en      (RX_FIFO_RE     ), // input rd_en
        .dout       (RX_FIFO_RD0[7:0]), // output [7 : 0] dout
        .valid      (andRX_FIFO_VAL[0]    )  // output valid
    );
    
    jtag_rx_fifo    JTAG_FIFO1 (
        .clk        (CLK            ), // input clk
        .rst        (RST            ), // input rst
        .full       (), // output full
		  .almost_full(orLOC_FULL[1]),
        .prog_full  (orLOC_PFULL[1]       ), // output prog_full
        .wr_en      (FIFO_WE[1]         ), // input wr_en
        .din        (exLOC_WD[7:0]    ), // input [7 : 0] din
        .empty      (orRX_FIFO_EMPTY[1]  ), // output empty
        .rd_en      (RX_FIFO_RE     ), // input rd_en
        .dout       (RX_FIFO_RD1[7:0]), // output [7 : 0] dout
        .valid      (andRX_FIFO_VAL[1]    )  // output valid
    );

    jtag_rx_fifo    JTAG_FIFO2 (
        .clk        (CLK            ), // input clk
        .rst        (RST            ), // input rst
        .full       (), // output full
		  .almost_full(orLOC_FULL[2]),
        .prog_full  (orLOC_PFULL[2]       ), // output prog_full
        .wr_en      (FIFO_WE[2]         ), // input wr_en
        .din        (exLOC_WD[7:0]    ), // input [7 : 0] din
        .empty      (orRX_FIFO_EMPTY[2]  ), // output empty
        .rd_en      (RX_FIFO_RE     ), // input rd_en
        .dout       (RX_FIFO_RD2[7:0]), // output [7 : 0] dout
        .valid      (andRX_FIFO_VAL[2]    )  // output valid
    );

    jtag_rx_fifo    JTAG_FIFO3 (
        .clk        (CLK            ), // input clk
        .rst        (RST            ), // input rst
        .full       (), // output full
		  .almost_full(orLOC_FULL[3]),
        .prog_full  (orLOC_PFULL[3]       ), // output prog_full
        .wr_en      (FIFO_WE[3]         ), // input wr_en
        .din        (exLOC_WD[7:0]    ), // input [7 : 0] din
        .empty      (orRX_FIFO_EMPTY[3]  ), // output empty
        .rd_en      (RX_FIFO_RE     ), // input rd_en
        .dout       (RX_FIFO_RD3[7:0]), // output [7 : 0] dout
        .valid      (andRX_FIFO_VAL[3]   )  // output valid
    );
*/
    jtag_rx_fifo_2048    JTAG_FIFO0 (
        .clk        (CLK            ), // input clk
        .rst        (RST            ), // input rst
        .full       (), // output full
		  .almost_full(orLOC_FULL[0]),
        .prog_full  (orLOC_PFULL[0]       ), // output prog_full
        .wr_en      (FIFO_WE[0]         ), // input wr_en
        .din        (exLOC_WD[7:0]    ), // input [7 : 0] din
        .empty      (orRX_FIFO_EMPTY[0]  ), // output empty
        .rd_en      (RX_FIFO_RE     ), // input rd_en
        .dout       (RX_FIFO_RD0[7:0]), // output [7 : 0] dout
        .valid      (andRX_FIFO_VAL[0]    )  // output valid
    );
    
    jtag_rx_fifo_2048    JTAG_FIFO1 (
        .clk        (CLK            ), // input clk
        .rst        (RST            ), // input rst
        .full       (), // output full
		  .almost_full(orLOC_FULL[1]),
        .prog_full  (orLOC_PFULL[1]       ), // output prog_full
        .wr_en      (FIFO_WE[1]         ), // input wr_en
        .din        (exLOC_WD[7:0]    ), // input [7 : 0] din
        .empty      (orRX_FIFO_EMPTY[1]  ), // output empty
        .rd_en      (RX_FIFO_RE     ), // input rd_en
        .dout       (RX_FIFO_RD1[7:0]), // output [7 : 0] dout
        .valid      (andRX_FIFO_VAL[1]    )  // output valid
    );

    jtag_rx_fifo_2048    JTAG_FIFO2 (
        .clk        (CLK            ), // input clk
        .rst        (RST            ), // input rst
        .full       (), // output full
		  .almost_full(orLOC_FULL[2]),
        .prog_full  (orLOC_PFULL[2]       ), // output prog_full
        .wr_en      (FIFO_WE[2]         ), // input wr_en
        .din        (exLOC_WD[7:0]    ), // input [7 : 0] din
        .empty      (orRX_FIFO_EMPTY[2]  ), // output empty
        .rd_en      (RX_FIFO_RE     ), // input rd_en
        .dout       (RX_FIFO_RD2[7:0]), // output [7 : 0] dout
        .valid      (andRX_FIFO_VAL[2]    )  // output valid
    );

    jtag_rx_fifo_2048    JTAG_FIFO3 (
        .clk        (CLK            ), // input clk
        .rst        (RST            ), // input rst
        .full       (), // output full
		  .almost_full(orLOC_FULL[3]),
        .prog_full  (orLOC_PFULL[3]       ), // output prog_full
        .wr_en      (FIFO_WE[3]         ), // input wr_en
        .din        (exLOC_WD[7:0]    ), // input [7 : 0] din
        .empty      (orRX_FIFO_EMPTY[3]  ), // output empty
        .rd_en      (RX_FIFO_RE     ), // input rd_en
        .dout       (RX_FIFO_RD3[7:0]), // output [7 : 0] dout
        .valid      (andRX_FIFO_VAL[3]   )  // output valid
    );

	 wire		andRX_FIFO_EMPTY;

    assign RX_FIFO_EMPTY = orRX_FIFO_EMPTY[0] | orRX_FIFO_EMPTY[1] | orRX_FIFO_EMPTY[2] | orRX_FIFO_EMPTY[3];
	 assign andRX_FIFO_EMPTY = (orRX_FIFO_EMPTY[3:0] == 4'b1111);
    assign RX_FIFO_VAL = (andRX_FIFO_VAL[3:0] == 4'b1111);
	 assign LOC_FULL = orLOC_FULL[0] | orLOC_FULL[1] | orLOC_FULL[2] | orLOC_FULL[3];
	 assign LOC_PFULL = orLOC_PFULL[0] | orLOC_PFULL[1] | orLOC_PFULL[2] | orLOC_PFULL[3];

	 reg	rxFifoRe;

    always @(posedge CLK)begin
        rxFifoRe <= ~RX_FIFO_EMPTY & jtagEnb;
    end

    assign  RX_FIFO_RE = rxFifoRe & RD_ENB;

    reg     [31:0]   rxFifoRd        ;
	 	 
    always @(posedge CLK)begin
		  if(RST) begin
				rxFifoRd[31:0] <= 32'd0;
        end else if(RX_FIFO_VAL)begin
            rxFifoRd[7:0]   <= RX_FIFO_RD0[7:0];
				rxFifoRd[15:8]  <= RX_FIFO_RD1[7:0];
				rxFifoRd[23:16] <= RX_FIFO_RD2[7:0];
				rxFifoRd[31:24] <= RX_FIFO_RD3[7:0];
        end 
    end

    (* IOB ="TRUE" *) reg     [31:0]   orPOUT   ;
	 
    always @(posedge CLK)begin
        orPOUT[31:0] <= rxFifoRd[31:0];
	 end

    assign  POUT[31:0] = orPOUT[31:0];
	 
	 wire			orRD_ENB_OFF;
	 reg        dlyRX_FIFO_EMPTY;
	 
	 always @(posedge CLK)begin
        dlyRX_FIFO_EMPTY <= andRX_FIFO_EMPTY;
    end
	 
	 assign orRD_ENB_OFF = ~dlyRX_FIFO_EMPTY & andRX_FIFO_EMPTY;
	 assign RD_ENB_OFF = orRD_ENB_OFF;

endmodule



