/*******************************************************************************
*                                                                              *
* System      : PulGen top module (Ver.2)                                      *
* Block       : ----                                                           *
* Module      : ---                                                            *
* Version     : v 0.0.0 2013/12/12                                             *
*                                                                              *
* Description : Top Module                                                     *
*                                                                              *
* Designer    : Tomohisa Uchida                                                *
*                                                                              *
*                Copyright (c) 2010 Tomohisa Uchida                            *
*                All rights reserved                                           *
*                                                                              *
*******************************************************************************/
`define FPGA_VER	32'h1602_1201
`define SYN_DATE    32'h1602_1201

`define PACKET_HD_RAW_TYPE 8'h02
`define PACKET_HD_SP_TYPE 8'h01
`define PACKET_HD_VER 8'h06

//`include "../CoreGen/sys_dcm.v"

//`include "JTAG_TOP.V"
//`include "JTAG.V"
//`include "JTAG_clk.V"
//`include "JTAG_mem.V"
//`include "counter.V"

`include "LOC_REG.v"
//`include "ZDLM_SYSCLK.V"

//`include "../CoreGen/zdlm_mem_tdc.v"
//`include "../CoreGen/ZDLM_SYS.v"

//`include "TIMER.v"
`include "WRAP_SiTCP_MII_XC3A_8K.V"
//`include "mii_init.v"

module PulGen_top(
// System
	OSC50M				,	// in	: System clock 50MHz
//JTAG
	POUT					,
	JTAGENB_OUT       ,

// SFP
//	SFP_REF_CLK_P		,	// in	: Reference Clock
//	SFP_REF_CLK_N		,	// in	: Reference Clock
//	SFP_TXP				,	// out	: Tx signal line
//	SFP_TXN				,	// out	: 
//	SFP_RXP				,	// in	: Rx signal line (Inverted)
//	SFP_RXN				,	// in	: 

// MII interface
	MII_RSTn			,	
	// TX
	MII_TX_CLK			,	// in	: Tx clock(2.5 or 25MHz)
	MII_TX_EN			,	// out	: Tx enable
	MII_TXD				,	// out	: Tx data[3:0]
	MII_TX_ER			,	// out	: TX error
	// RX
	MII_RX_CLK			,	// in	: Rx clock(2.5 or 25MHz)
	MII_RX_DV			,	// in	: Rx data valid
	MII_RXD				,	// in	: Rx data[3:0]
	MII_RX_ER			,	// in	: Rx error
	MII_CRS				,	// in	: Carrier sense
	MII_COL				,	// in	: Collision detected
	// Management IF
	MII_MDC				,	// out	: Clock for MDIO
	MII_MDIO			,	// in	: data in
	
	MII_PHY_CLK		,
	
	FORCE_DEFAULTn			,
	
	MII_LED_FULL,
	MII_LED_ACT,
	MII_LED_LINK,
	MII_LED_1000M,
	
// EEPROM
	EEP_SK				,	// out	: Clock
	EEP_CS				,	// out	: Chip select
	EEP_DI				,	// out	: Data out
	EEP_DO				,	// in	: Data in
	
	TESTPIN 				,	// in	: TESTPIN[3:0]
	SM_LED					// out	: LED[3:0]

);

//-------- Input/Output -------------
	input			OSC50M			;
	
	output		JTAGENB_OUT;

//	input			SFP_REF_CLK_P	;
//	input			SFP_REF_CLK_N	;
//	output			SFP_TXP			;
//	output			SFP_TXN			;
//	input			SFP_RXP			;
//	input			SFP_RXN			;

	output			MII_RSTn		;
	input			MII_TX_CLK		;
	output			MII_TX_EN		;
	output	[3:0]	MII_TXD			;
	output			MII_TX_ER		;
	input			MII_RX_CLK		;
	input			MII_RX_DV		;
	input	[3:0]	MII_RXD			;
	input			MII_RX_ER		;
	input			MII_CRS			;
	input			MII_COL			;
	output			MII_MDC			;
	inout			MII_MDIO		;
	
	output		MII_PHY_CLK		;
	
	input			FORCE_DEFAULTn	;
	
	output		MII_LED_FULL;
	output		MII_LED_ACT;
	output		MII_LED_LINK;
	output		MII_LED_1000M;

	output			EEP_SK		;
	output			EEP_CS		;
	output			EEP_DI		;
	input			EEP_DO		;
	
	output	[31:0]	POUT				;
	
	input	[3:0]	TESTPIN 			;
	output	[2:0]	SM_LED			;
	
//------------------------------------------------------------------------------
//	Buffers
//------------------------------------------------------------------------------

//	wire			SFP_TXP			;
//	wire			SFP_TXN			;
	
	wire			EEP_SK		;
	wire			EEP_CS		;
	wire			EEP_DI		;
	
	wire	[3:0]	SM_LED			;
	
	wire			MII_MDOE		;
	wire			MII_MDIO		;
	wire			MII_MDOUT		;

	assign	MII_MDIO	= (MII_MDOE	? MII_MDOUT : 1'bz);
	
//------------------------------------------------------------------------------
//	System management
//------------------------------------------------------------------------------
	wire			IBUF_CLK50M		;

	IBUFG    IBUFG50M (.O( IBUF_CLK50M), .I(OSC50M));

	wire			oscFds			;
	wire	[2:0]	oscFd			;

	FDS	OSC_FDS(.D(1'b0), .C(IBUF_CLK50M), .Q(oscFds), .S(1'b0)); 
	FD	OSC_FD_0(.D(oscFds), .C(IBUF_CLK50M), .Q(oscFd[0])); 
	FD	OSC_FD_1(.D(oscFd[0]), .C(IBUF_CLK50M), .Q(oscFd[1])); 
	FD	OSC_FD_2(.D(oscFd[1]), .C(IBUF_CLK50M), .Q(oscFd[2])); 

	wire		sysDcmRst	;

	assign	sysDcmRst= (|oscFd[2:0]);

	wire			SYSCLK			;
	wire			DCM_SYS_LOCK	;
	wire			DCM_SYS_LOCK2	;
	wire			clk50M			;
//	wire			clk200M			;

	sys_dcm		sys_dcm(
		.RST_IN			(sysDcmRst		),
		.CLKIN_IN		(IBUF_CLK50M	),
		.CLKFX_OUT		(SYSCLK),
		.CLKIN_IBUFG_OUT		(clk50M			),
		.LOCKED_OUT			(DCM_SYS_LOCK	)
	);
/*
	sys_dcm2		sys_dcm2(
		.RST_IN			(sysDcmRst		),
		.CLKIN_IN		(clk50M	),
		.CLKFX_OUT		(clk200M),
		.CLKIN_IBUFG_OUT		(			),
		.LOCKED_OUT			(DCM_SYS_LOCK2	)
	);
*/	

	reg			RST				;
	
	always @ (posedge SYSCLK) begin
		RST <= ~DCM_SYS_LOCK; //& ~DCM_SYS_LOCK2;
	end
	
	OFDDRCPE clk_25M(.D0(1'b1), .D1(1'b0), .C0(SYSCLK), .C1(~SYSCLK), .CLR(RST), .CE(1'b1), .PRE(1'b0), .Q(MII_PHY_CLK));

	assign MII_LED_FULL = 1'bZ;
	assign MII_LED_ACT = 1'bZ;
	assign MII_LED_LINK = 1'bZ;
	assign MII_LED_1000M = 1'bZ;
	
//------------------------------------------------------------------------------
//	LED
//------------------------------------------------------------------------------
	reg	[31:0]	ledCnt;

	always@ (posedge SYSCLK or posedge RST) begin
		if(RST)begin
			ledCnt[31:0] <= 32'd0;
		end else begin
			ledCnt[31:0] <= ledCnt[31:0] + 32'd1;
		end
	end
	
	wire			MII_RSTn;
	wire			SiTCP_RST;
	
//	assign MII_RSTn = ~MII_RESETn;
	
//	wire			SiTCP_ACTIVE		;
//	wire			resetdone;

//	assign	SM_LED[3]	= ledCnt[24];
	assign	SM_LED[0]	= ledCnt[24];
	assign	SM_LED[1]	= MII_RSTn;
	assign	SM_LED[2]	= SiTCP_RST;

//------------------------------------------------------------------------------
//	NETWORK PROTOCOL PROCESSOR
//------------------------------------------------------------------------------

	wire				TCP_OPEN				;
	wire				TCP_CLOSE				;
//	wire				TCP_TX_WR			;
	wire	[15:0]	TCP_RX_WC			;
	wire	[7:0]	TCP_RX_DATA			;
//	wire	[7:0]	TCP_TX_DATA			;

	wire	[31:0]	RBCP_ADDR			;
	wire 			RBCP_WE				;
	wire	[7:0]	RBCP_WD				;
	wire 			RBCP_RE				;
	wire 			RBCP_ACK			;
	wire	[7:0]	RBCP_RD				;

	WRAP_SiTCP_MII_XC3A_8K	
		#(25) // = System clock frequency(MHz)
	SiTCP(
		.CLK					(SYSCLK				),	// in	: System Clock (MII: >15MHz, GMII=150MHz)
		.RST					(RST				),	// in	: System reset
		.FORCE_DEFAULTn			(1'd0),	// in	: Load default values
		.EXT_IP_ADDR			(32'd0),	// in	: My IP address[31:0]
		.EXT_TCP_PORT			(16'd0),	// in	: My TCP main-port #[15:0]
		.EXT_RBCP_PORT			(16'd0),	// in	: My TCP sub-port #[15:0]
		.PHY_ADDR				(5'b01111				),	// in	: PHY-device MIF address[4:0]
	// user data, intialial values are stored in the EEPROM, 0xFFFF_FC3C-3F
		.USR_REG_X3C			(),	// out	: Stored at 0xFFFF_FF3C
		.USR_REG_X3D			(),	// out	: Stored at 0xFFFF_FF3D
		.USR_REG_X3E			(),	// out	: Stored at 0xFFFF_FF3E
		.USR_REG_X3F			(),	// out	: Stored at 0xFFFF_FF3F
	// EEPROM
		.EEPROM_CS				(EEP_CS),	// out	: Chip select
		.EEPROM_SK				(EEP_SK),	// out	: Serial data clock
		.EEPROM_DI				(EEP_DI),	// out	: Serial write data
		.EEPROM_DO				(EEP_DO),	// in	: Serial read data
	// MII interface
		.MII_RSTn				(MII_RSTn			),	// out	: PHY reset
		.MII_1000M				(1'd0),	// in	: GMII mode (0:MII, 1:GMII)
		// TX
		.MII_TX_CLK				(MII_TX_CLK			),	// in	: Tx clock(2.5 or 25MHz)
		.MII_TX_EN				(MII_TX_EN			),	// out	: Tx enable
		.MII_TXD				(MII_TXD[3:0]		),	// out	: Tx data[3:0]
		.MII_TX_ER				(MII_TX_ER			),	// out	: TX error
		// RX
		.MII_RX_CLK				(MII_RX_CLK			),	// in	: Rx clock(2.5 or 25MHz)
		.MII_RX_DV				(MII_RX_DV			),	// in	: Rx data valid
		.MII_RXD				(MII_RXD[3:0]		),	// in	: Rx data[3:0]
		.MII_RX_ER				(MII_RX_ER			),	// in	: Rx error
		.MII_CRS				(MII_CRS			),	// in	: Carrier sense
		.MII_COL				(MII_COL			),	// in	: Collision detected
		// Management IF
		.MII_MDC				(MII_MDC			),	// out	: Clock for MDIO
		.MII_MDIO_IN			(MII_MDIO			),	// in	: Data
		.MII_MDIO_OUT			(MII_MDOUT			),	// out	: Data
		.MII_MDIO_OE			(MII_MDOE			),	// out	: MDIO output enable
	// User I/F
		.SiTCP_RST				(SiTCP_RST			),	// out	: Reset for SiTCP and related circuits
		// TCP connection control
		.TCP_OPEN_REQ			(1'b0),	// in	: Reserved input, shoud be 0
		.TCP_OPEN_ACK			(TCP_OPEN			),	// out	: Acknowledge for open (=Socket busy)
		.TCP_ERROR				(),	// out	: TCP error, its active period is equal to MSL
		.TCP_CLOSE_REQ			(TCP_CLOSE			),	// out	: Connection close request
		.TCP_CLOSE_ACK			(TCP_CLOSE			),	// in	: Acknowledge for closing
		// FIFO I/F
		.TCP_RX_WC				(TCP_RX_WC[15:0]	),	// in	: Rx FIFO write count[15:0] (Unused bits should be set 1)
		.TCP_RX_WR				(TCP_RX_WR			),	// out	: Write enable
		.TCP_RX_DATA			(TCP_RX_DATA[7:0]	),	// out	: Write data[7:0]
		.TCP_TX_FULL			(TCP_TX_FULL		),	// out	: Almost full flag
		.TCP_TX_WR				(1'b0			),	// in	: Write enable
		.TCP_TX_DATA			(8'd0	),	// in	: Write data[7:0]
		// RBCP
		.RBCP_ACT				(RBCP_ACT			),	// out	: RBCP active
		.RBCP_ADDR				(RBCP_ADDR[31:0]	),	// out	: Address[31:0]
		.RBCP_WD				(RBCP_WD[7:0]		),	// out	: Data[7:0]
		.RBCP_WE				(RBCP_WE			),	// out	: Write enable
		.RBCP_RE				(RBCP_RE			),	// out	: Read enable
		.RBCP_ACK				(RBCP_ACK			),	// in	: Access acknowledge
		.RBCP_RD				(RBCP_RD[7:0]		)	// in	: Read data[7:0]
	);
	
	wire	[5:0]	JTAG_ENB;

	wire		JTAG_WE;
	wire	[7:0]	JTAG_WD;
	wire		JTAG_RD_ENB;
	wire	[7:0]	JTAG_FC;
	
	wire		RD_ENB_OFF;
	wire		RD_ENB_OFF_50M;
//	wire		RD_ENB_OFF_200M;
	
	wire		JTAGENB_OUT;
	wire		JTAGENB_OUT_50M;
//	wire		JTAGENB_OUT_200M;
	
	
	wire [31:0]	POUT_50M;	
	wire		JTAG_FULL_50M;
	wire		JTAG_PFULL_50M;
	
//	wire [31:0]	POUT_200M;
//	wire		JTAG_FULL_200M;
//	wire		JTAG_PFULL_200M;
	
//	wire 			CLK200M_EN;

	wire [31:0]	POUT;
	wire		JTAG_FULL;
	wire		JTAG_PFULL;
/*	
	assign POUT[31:0]  = CLK200M_EN ? POUT_200M[31:0]  : POUT_50M[31:0];
	assign JTAG_FULL   = CLK200M_EN ? JTAG_FULL_200M   : JTAG_FULL_50M;
	assign JTAG_PFULL  = CLK200M_EN ? JTAG_PFULL_200M  : JTAG_PFULL_50M;
	assign RD_ENB_OFF  = CLK200M_EN ? RD_ENB_OFF_200M  : RD_ENB_OFF_50M;
	assign JTAGENB_OUT = CLK200M_EN ? JTAGENB_OUT_200M : JTAGENB_OUT_50M;
*/	
	assign POUT[31:0]  = POUT_50M[31:0];
	assign JTAG_FULL   = JTAG_FULL_50M;
	assign JTAG_PFULL  = JTAG_PFULL_50M;
	assign RD_ENB_OFF  = RD_ENB_OFF_50M;
	assign JTAGENB_OUT = JTAGENB_OUT_50M;
	
    LOC_REG LOC_REG(
    .CLK                 (clk50M			),   // in   : Clock
	 //.CLK                 (SYSCLK			),   // in   : Clock
    .RST                 (SiTCP_RST				),   // in   : System reset
    // System monitor
    .SMON_DADDR          (),   // out  : address[6:0]
    .SMON_DEN            (),   // out  : Device enable
    .SMON_DWE            (),   // out  : Write enable
    .SMON_DWD            (),   // out  : Write data[15:0]
    .SMON_DRD            (16'd0),   // in   : Read data[15:0]
    .SMON_RDY            (1'b0),   // in   : Ready
    // SiTCP RBCP I/F
    .LOC_ADDR            (RBCP_ADDR			),   // in   : Address[31:0]
    .LOC_WD              (RBCP_WD			),   // in   : Data[7:0]
    .LOC_WE              (RBCP_WE			),   // in   : Write enable
    .LOC_RE              (RBCP_RE			),   // in   : Read enable
    .LOC_ACK             (RBCP_ACK			),   // out  : Access acknowledge
    .LOC_RD              (RBCP_RD			),   // out  : Read data[7:0]
    // JTAG I/F
    .JTAG_ENB            (JTAG_ENB),   // out  : JTAG I/F enable for each FEEs
    .JTAG_FULL           (JTAG_FULL),   // in   : JTAG FIFO full flag
	 .JTAG_PFULL          (JTAG_PFULL      ),   // out  : 
	 .JTAG_RD_ENB			 (JTAG_RD_ENB			),
	 .JTAG_RD_ENB_OFF			 (RD_ENB_OFF			),
    .JTAG_WE             (JTAG_WE),   // out  : Write enable for JTAG data
    .JTAG_WD             (JTAG_WD),   // out  : Write data
	 .JTAG_FC				 (JTAG_FC),
	 .CLK200M_EN			 (			),
    // Arich registers
    .FEE_DATA_WIDTH4_8       ()// out  : Data width of FEE data (0:4-bit, 1:8-bit)
    );
	
	JTAG_IF JTAG_IF(
    //s.CLK                 (SYSCLK			),   // in   : Clock
	 .CLK                 (clk50M			),   // in   : Clock
    .RST                 (SiTCP_RST		),   // in   : System reset
    // JTAG
    .POUT                 (POUT_50M            ),   // out  :
	 .RD_ENB					  (JTAG_RD_ENB),
	 .JTAG_FC					 (JTAG_FC),
	 .JTAGENB_OUT			 (JTAGENB_OUT_50M),
	 .RD_ENB_OFF			 (RD_ENB_OFF_50M),
    // Slow control I/F
    .LOC_ENB             (JTAG_ENB       ),   // in   : JTAG I/F enable for each FEEs
    .LOC_FULL            (JTAG_FULL_50M      ),   // out  : 
	 .LOC_PFULL            (JTAG_PFULL_50M      ),   // out  : 
    .LOC_WE              (JTAG_WE        ),   // in   : Write enable
    .LOC_WD              (JTAG_WD        )// in   : Write data[7:0]
	);
/*	
	JTAG_IF JTAG_IF2(
    //s.CLK                 (SYSCLK			),   // in   : Clock
	 .CLK                 (clk200M			),   // in   : Clock
    .RST                 (SiTCP_RST		),   // in   : System reset
    // JTAG
    .POUT                 (POUT_200M            ),   // out  :
	 .RD_ENB					  (JTAG_RD_ENB),
	 .JTAG_FC					 (JTAG_FC),
	 .JTAGENB_OUT			 (JTAGENB_OUT_200M),
	 .RD_ENB_OFF			 (RD_ENB_OFF_200M),
    // Slow control I/F
    .LOC_ENB             (JTAG_ENB       ),   // in   : JTAG I/F enable for each FEEs
    .LOC_FULL            (JTAG_FULL_200M      ),   // out  : 
	 .LOC_PFULL            (JTAG_PFULL_200M      ),   // out  : 
    .LOC_WE              (JTAG_WE        ),   // in   : Write enable
    .LOC_WD              (JTAG_WD        )// in   : Write data[7:0]
	);
*/	
/*******************************************************************************
*     SFP Interface                                                            *
*******************************************************************************/
/*
//	wire		sfpRefClk	;

//	IBUFDS	IBUF_SFP_REF (.I(SFP_REF_CLK_P), .IB(SFP_REF_CLK_N), .O(sfpRefClk));

//	assign	GMII_1000M	= 1'b1;

	wire	mdc				;
	wire	mdio_out		;

	mii_initializer		mii_initializer(
		// System
		.CLK				(clk125m			),  // in : system clock (125M)
		.RST				(RST				), // in : system reset
		// PHY
		.PHYAD				(5'd0					), // in : [4:0] PHY address
		// MII
		.MDC				(mdc				), // out: clock (1/128 system clock)
		.MDIO_OUT			(mdio_out			), // out: connect this to "PCS/PMA + RocketIO" module .mdio?_i()
		// status
		.COMPLETE			(mii_complete		)  // out: initializing sequence has completed (active H)
	);

	gig_eth_pcs_pma_v11_5_example_design	gig_eth_pcs_pma_v11_5_example_design(
      // GMII Interface
      //---------------
      .gmii_txd0         (ETH_TX_D[7:0]  ),  // in  : Transmit data from client MAC. [7:0]
      .gmii_tx_en0       (ETH_TX_EN     ),  // in  : Transmit control signal from client MAC.
      .gmii_tx_er0       (ETH_TX_ER     ),  // in  : Transmit control signal from client MAC.
      .gmii_rxd0         (ETH_RX_D[7:0]  ),  // out : Received Data to client MAC. [7:0]
      .gmii_rx_dv0       (ETH_RX_DV     ),  // out : Received control signal to client MAC.
      .gmii_rx_er0       (ETH_RX_ER     ),  // out : Received control signal to client MAC.

      // Management: MDIO Interface
      //---------------------------
      .mdc0                  (mdc     ),  // in  : Management Data Clock
      .mdio0_i               (mdio_out  ),  // in  : Management Data In
      .mdio0_o               (),                 // out : Management Data Out
      .mdio0_t               (),                 // out : Management Data Tristate
      .phyad0                (5'b00001       ),  // in  : Port address for MDIO. [4:0]
      .configuration_vector0 (5'b00000       ),  // in  : Alternative to MDIO interface. 4bit
      .configuration_valid0  (1'b0           ),  // in  : Validation signal for Config vector. 1bit

      // General IO's
      //-------------
      .link_timer_value0     (9'b100111101            ),  // in  : Programmable Auto-Negotiation Link Timer Control [8:0]
      .an_interrupt0         (),                          // out : Interrupt to processor to signal that Auto-Negotiation has completed
      .an_adv_config_vector0 (16'b0000_0000_0010_0001 ),  // in  : Alternate interface to program REG4 (AN ADV)
      .an_adv_config_val0    (1'b0                    ),  // in  : Validation signal for AN ADV
      .an_restart_config0    (1'b0                    ),  // in  : Alternate signal to modify AN restart bit in REG0
      .status_vector0        (),                          // out : Core status.
      .reset0                (RST         ),  // in  : Asynchronous reset for entire core.
      .signal_detect0        (1'b1                    ),  // in  : Input from PMD to indicate presence of optical input.
      .gmii_isolate0         (gmii_isolate          ),  // out : internal gmii_isolate signal.

      .clk125m           (clk125m         ),  // out : Ether System Clock 125MHz
      .resetdone0         (resetdone       ), 

      //------------------------------------------------------------------------
      // Tranceiver interfaces
      //------------------------------------------------------------------------

      .brefclk_p (SFP_REF_CLK_P  ),  // in  : Differential +ve of reference clock for tranceiver: 125MHz, very high quality
      .brefclk_n (SFP_REF_CLK_N  ),  // in  : Differential -ve of reference clock for tranceiver: 125MHz, very high quality

      .txp0      (SFP_TXP     ),  // out : Differential +ve of serial transmission from PMA to PMD.
      .txn0      (SFP_TXN     ),  // out : Differential -ve of serial transmission from PMA to PMD.
      .rxp0      (SFP_RXP     ),  // in  : Differential +ve for serial reception from PMD to PMA.
      .rxn0      (SFP_RXN     ),  // in  : Differential -ve for serial reception from PMD to PMA.

      .txp1      (     ),  // out : Differential +ve of serial transmission from PMA to PMD.
      .txn1      (     ),  // out : Differential -ve of serial transmission from PMA to PMD.
      .rxp1      (1'd0     ),  // in  : Differential +ve for serial reception from PMD to PMA.
      .rxn1      (1'd1     )   // in  : Differential -ve for serial reception from PMD to PMA.
   );
*/
endmodule
//-------------------------------------------------------------------
